/* BdfServer - Copyright (c) 2009-2023 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.tools.balayage.engine;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.balayage.Balayage;
import fr.exemole.bdfserver.api.balayage.BalayageMode;
import fr.exemole.bdfserver.api.balayage.FileGenerationListener;
import fr.exemole.bdfserver.api.configuration.PathConfiguration;
import fr.exemole.bdfserver.tools.BdfServerUtils;
import fr.exemole.bdfserver.tools.configuration.ConfigurationUtils;
import java.io.File;
import java.util.List;
import javax.xml.transform.URIResolver;
import net.fichotheque.corpus.Fiches;
import net.fichotheque.corpus.SortConstants;
import net.fichotheque.extraction.ExtractionContext;
import net.fichotheque.selection.FicheQuery;
import net.fichotheque.selection.SelectionContext;
import net.fichotheque.tools.corpus.FichesBuilder;
import net.fichotheque.tools.permission.PermissionUtils;
import net.fichotheque.utils.EligibilityUtils;
import net.fichotheque.utils.selection.FicheSelectorBuilder;
import net.fichotheque.xml.extraction.ExtractParameters;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.LangContext;


/**
 *
 * @author Vincent Calame
 */
public class BalayageParameters {

    private final Balayage balayage;
    private final BdfServer bdfServer;
    private final PathConfiguration pathConfiguration;
    private ExtractParameters extractParameters = null;
    private BalayageMode balayageMode;
    private BalayageLog balayageLog;
    private File balayageRootDirectory;
    private FileGenerationListener fileGenerationListener;
    private URIResolver customURIResolver;
    private Fiches selectedFiches;

    public BalayageParameters(Balayage balayage, BdfServer bdfServer, PathConfiguration pathConfiguration) {
        this.balayage = balayage;
        this.bdfServer = bdfServer;
        this.pathConfiguration = pathConfiguration;
    }

    public BalayageMode getBalayageMode() {
        return balayageMode;
    }

    public void setBalayageMode(BalayageMode balayageMode) {
        this.balayageMode = balayageMode;
    }

    public BalayageLog getBalayageLog() {
        return balayageLog;
    }

    public void setBalayageLog(BalayageLog balayageLog) {
        this.balayageLog = balayageLog;
    }

    public Balayage getBalayage() {
        return balayage;
    }

    public BdfServer getBdfServer() {
        return bdfServer;
    }

    public PathConfiguration getPathConfiguration() {
        return pathConfiguration;
    }

    public File getBalayageRootDirectory() {
        return balayageRootDirectory;
    }

    public void setBalayageRootDirectory(File balayageRootDirectory) {
        this.balayageRootDirectory = balayageRootDirectory;
    }

    public ExtractParameters getExtractParameters() {
        if (extractParameters != null) {
            return extractParameters;
        }
        initExtractParameters();
        return extractParameters;
    }

    public Fiches getSelectedFiches() {
        if (extractParameters != null) {
            return selectedFiches;
        }
        initExtractParameters();
        return selectedFiches;
    }

    /**
     * @return the fileGenerationListener
     */
    public FileGenerationListener getFileGenerationListener() {
        return fileGenerationListener;
    }

    /**
     * @param fileGenerationListener the fileGenerationListener to set
     */
    public void setFileGenerationListener(FileGenerationListener fileGenerationListener) {
        this.fileGenerationListener = fileGenerationListener;
    }

    public URIResolver getCustomURIResolver() {
        return customURIResolver;
    }

    public void setCustomURIResolver(URIResolver customURIResolver) {
        this.customURIResolver = customURIResolver;
    }

    private void initExtractParameters() {
        LangContext langContext = balayage.getLangContext();
        if (langContext == null) {
            langContext = ConfigurationUtils.toLangContext(bdfServer.getLangConfiguration(), false);
        }
        this.selectedFiches = initSelectedFiches(langContext);
        ExtractionContext extractionContext = BdfServerUtils.initExtractionContextBuilder(bdfServer, langContext, PermissionUtils.FICHOTHEQUEADMIN_PERMISSIONSUMMARY).toExtractionContext();
        this.extractParameters = new ExtractParameters(extractionContext, balayage.getExtractVersion(), false, selectedFiches);
    }

    private Fiches initSelectedFiches(LangContext langContext) {
        Fiches fiches = null;
        List<FicheQuery> ficheQueryList = balayage.getGlobalFicheQueryList();
        if (!ficheQueryList.isEmpty()) {
            Lang defaultLang = langContext.getDefaultLang();
            SelectionContext selectionContext = BdfServerUtils.initSelectionContextBuilder(bdfServer, defaultLang)
                    .setSubsetAccessPredicate(EligibilityUtils.ALL_SUBSET_PREDICATE)
                    .toSelectionContext();
            FicheSelectorBuilder ficheSelectorBuilder = FicheSelectorBuilder.init(selectionContext);
            for (FicheQuery ficheQuery : ficheQueryList) {
                ficheSelectorBuilder.add(ficheQuery);
            }
            fiches = FichesBuilder.build(SortConstants.NONE)
                    .populate(ficheSelectorBuilder.toFicheSelector())
                    .toFiches();
        }
        return fiches;
    }

}
