/* BdfServer_Commands - Copyright (c) 2015-2023 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands.sphere;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.BdfServerEditor;
import fr.exemole.bdfserver.api.EditSession;
import fr.exemole.bdfserver.api.exportation.table.FicheTableParameters;
import fr.exemole.bdfserver.api.exportation.table.TableExportParameters;
import fr.exemole.bdfserver.api.interaction.Domains;
import fr.exemole.bdfserver.api.interaction.InteractionConstants;
import fr.exemole.bdfserver.api.namespaces.BdfUserSpace;
import fr.exemole.bdfserver.api.users.BdfUserPrefs;
import fr.exemole.bdfserver.tools.exportation.table.BdfTableExportUtils;
import fr.exemole.bdfserver.tools.exportation.table.TableExportParametersBuilder;
import fr.exemole.bdfserver.tools.instruction.AbstractBdfCommand;
import fr.exemole.bdfserver.tools.instruction.BdfErrors;
import fr.exemole.bdfserver.tools.interaction.FicheTableParametersParser;
import fr.exemole.bdfserver.tools.users.BdfUserUtils;
import java.util.List;
import net.fichotheque.sphere.Redacteur;
import net.fichotheque.utils.TableExportUtils;
import net.mapeadores.util.attr.Attribute;
import net.mapeadores.util.attr.AttributeBuilder;
import net.mapeadores.util.logging.ErrorMessageException;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public class RedacteurDefaultTableExportParametersCommand extends AbstractBdfCommand {

    public final static String COMMANDNAME = "RedacteurDefaultTableExportParameters";
    public final static String COMMANDKEY = "_ USR-05";
    public final static String STATUSSHEET_PARAMNAME = "statussheet";
    private BdfUserPrefs redacBdfUserPrefs;
    private TableExportParameters tableExportParameters;
    private boolean withStatusSheet;

    public RedacteurDefaultTableExportParametersCommand(BdfServer bdfServer, RequestMap requestMap) {
        super(bdfServer, requestMap);
    }

    @Override
    public boolean needSynchronisation() {
        return true;
    }

    @Override
    protected void doCommand() throws ErrorMessageException {
        try (EditSession session = startEditSession(Domains.SPHERE, COMMANDNAME)) {
            BdfServerEditor bdfServerEditor = session.getBdfServerEditor();
            Attribute headerTypeAttribute = AttributeBuilder.toAttribute(BdfUserSpace.TABLEEXPORT_HEADERTYPE_KEY, TableExportUtils.headerTypeToString(tableExportParameters.getHeaderType()));
            bdfServerEditor.putAttribute(redacBdfUserPrefs, headerTypeAttribute);
            Attribute patternModeAttribute = AttributeBuilder.toAttribute(BdfUserSpace.TABLEEXPORT_PATTERNMODE_KEY, BdfTableExportUtils.patternModeToString(tableExportParameters.getDefaulFicheTableParameters().getPatternMode()));
            bdfServerEditor.putAttribute(redacBdfUserPrefs, patternModeAttribute);
            List<String> list = tableExportParameters.getDefaulFicheTableParameters().getWithList();
            boolean withDone = false;
            if (!list.isEmpty()) {
                AttributeBuilder builder = new AttributeBuilder(BdfUserSpace.TABLEEXPORT_WITH_KEY);
                for (String value : list) {
                    builder.addValue(value);
                }
                Attribute withAttribute = builder.toAttribute();
                if (withAttribute != null) {
                    withDone = true;
                    bdfServerEditor.putAttribute(redacBdfUserPrefs, withAttribute);
                }
            }
            if (!withDone) {
                bdfServerEditor.removeAttribute(redacBdfUserPrefs, BdfUserSpace.TABLEEXPORT_WITH_KEY);
            }
            if (withStatusSheet) {
                Attribute statussheetAttribute = AttributeBuilder.toAttribute(BdfUserSpace.TABLEEXPORT_STATUSSHEET_KEY, "1");
                bdfServerEditor.putAttribute(redacBdfUserPrefs, statussheetAttribute);
            } else {
                bdfServerEditor.removeAttribute(redacBdfUserPrefs, BdfUserSpace.TABLEEXPORT_STATUSSHEET_KEY);
            }
        }
        setDone("_ done.sphere.redacteurdefaulttableexportparameters");
    }

    @Override
    protected void checkParameters() throws ErrorMessageException {
        Redacteur redacteur = getMandatoryRedacteur();
        boolean isSame = BdfUserUtils.isSame(bdfUser, redacteur);
        if (!isSame) {
            checkSubsetAdmin(redacteur.getSphere());
        }
        if (isSame) {
            redacBdfUserPrefs = bdfUser.getPrefs();
        } else {
            redacBdfUserPrefs = bdfServer.createBdfUser(redacteur).getPrefs();
        }
        TableExportParametersBuilder builder = new TableExportParametersBuilder();
        FicheTableParameters FicheTableParameters = FicheTableParametersParser.parse(requestMap);
        builder.setFicheTableParameters(FicheTableParameters);
        String headerTypeValue = requestMap.getParameter(InteractionConstants.HEADERTYPE_PARAMNAME);
        if (headerTypeValue != null) {
            try {
                short headerType = TableExportUtils.headerTypeToShort(headerTypeValue);
                builder.setHeaderType(headerType);
            } catch (IllegalArgumentException iae) {
                throw BdfErrors.unknownParameterValue(InteractionConstants.HEADERTYPE_PARAMNAME, headerTypeValue);
            }
        }
        tableExportParameters = builder.toTableExportParameters();
        this.withStatusSheet = requestMap.isTrue(STATUSSHEET_PARAMNAME);
    }

}
