/* FichothequeLib_Tools - Copyright (c) 2014-2018 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.tools.importation.directory;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.ArrayList;
import java.util.List;
import java.util.SortedSet;
import java.util.TreeSet;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.corpus.FicheChange;
import net.fichotheque.corpus.FicheMeta;
import net.fichotheque.corpus.fiche.ContentChecker;
import net.fichotheque.importation.CorpusImport;
import net.fichotheque.importation.LiensImport;
import net.fichotheque.tools.importation.ImportationUtils;
import net.fichotheque.tools.importation.Increment;
import net.fichotheque.tools.importation.dom.CorpusImportDOMReader;
import net.fichotheque.xml.importation.CorpusImportXMLPart;
import net.mapeadores.util.attr.AttributeChange;
import net.mapeadores.util.exceptions.NestedIOException;
import net.mapeadores.util.primitives.FuzzyDate;
import net.mapeadores.util.xml.AppendableXMLWriter;
import net.mapeadores.util.xml.DOMUtils;
import net.mapeadores.util.xml.XMLUtils;
import org.w3c.dom.Element;


/**
 *
 * @author Vincent Calame
 */
class CreationDirectoryCorpusImport extends DirectoryCorpusImport {

    private final ContentChecker contentChecker;
    private final Corpus corpus;
    private final File dir;
    private final List<String> fileList = new ArrayList<String>();
    private final Increment increment = new Increment();

    CreationDirectoryCorpusImport(ContentChecker contentChecker, Corpus corpus, File dir) {
        this.contentChecker = contentChecker;
        this.corpus = corpus;
        this.dir = dir;
        File[] files = dir.listFiles();
        int length = files.length;
        SortedSet<String> sortedSet = new TreeSet<String>();
        for (int i = 0; i < length; i++) {
            File f = files[i];
            if (f.isDirectory()) {
                continue;
            }
            String name = f.getName();
            if (ImportationUtils.isValidFicheImportFileName(CorpusImport.CREATION_TYPE, name)) {
                sortedSet.add(name);
            }
        }
        fileList.addAll(sortedSet);
        ImportationUtils.updateIncrement(CorpusImport.CREATION_TYPE, fileList, increment);
    }

    @Override
    public Corpus getCorpus() {
        return corpus;
    }

    @Override
    public String getType() {
        return CorpusImport.CREATION_TYPE;
    }

    @Override
    public int getFicheImportCount() {
        return fileList.size();
    }

    @Override
    public FicheImport getFicheImport(int i) {
        String fileName = fileList.get(i);
        File f = new File(dir, fileName);
        Element rootElement = DOMUtils.readDocument(f).getDocumentElement();
        String idString = rootElement.getAttribute("id");
        int newId = -1;
        if (idString.length() > 0) {
            try {
                newId = Integer.parseInt(idString);
            } catch (NumberFormatException nfe) {
            }
        }
        CorpusImportDOMReader.FicheImportConsumer ficheImportHandler = CorpusImportDOMReader.newFicheImportHandler(corpus.getFichotheque(), contentChecker);
        DOMUtils.readChildren(rootElement, ficheImportHandler);
        return new InternalCreationFicheImport(newId, ficheImportHandler.getFicheChangeBuilder().toFicheChange(), ficheImportHandler.getAttributeChangeBuilder().toAttributeChange(), ficheImportHandler.getLienImportBuilder().toLiensImport(), ficheImportHandler.getCreationDate());
    }

    @Override
    public void saveFicheImport(CorpusImport.FicheImport ficheImport) {
        CorpusImport.CreationFicheImport creationFicheImport = (CorpusImport.CreationFicheImport) ficheImport;
        StringBuilder buf = new StringBuilder();
        buf.append(CorpusImport.CREATION_TYPE);
        buf.append('-');
        increment.appendTo(buf);
        buf.append(".xml");
        increment.increase();
        File f = new File(dir, buf.toString());
        try (Writer writer = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(f), "UTF-8"))) {
            AppendableXMLWriter xmlWriter = XMLUtils.toXMLWriter(writer);
            xmlWriter.appendXMLDeclaration();
            CorpusImportXMLPart corpusImportXMLPart = new CorpusImportXMLPart(xmlWriter);
            corpusImportXMLPart.addCreationFicheImport(creationFicheImport);
        } catch (IOException ioe) {
            throw new NestedIOException(ioe);
        }
    }

    @Override
    public void endSave() {
    }


    private static class InternalCreationFicheImport implements CorpusImport.CreationFicheImport {

        private final int newId;
        private final FicheChange ficheChange;
        private final AttributeChange attributeChange;
        private final LiensImport liensImport;
        private final FuzzyDate creationDate;

        private InternalCreationFicheImport(int newId, FicheChange ficheChange, AttributeChange attributeChange, LiensImport liensImport, FuzzyDate creationDate) {
            this.newId = newId;
            this.ficheChange = ficheChange;
            this.attributeChange = attributeChange;
            this.liensImport = liensImport;
            this.creationDate = creationDate;
        }

        @Override
        public FicheMeta getFicheMeta() {
            return null;
        }

        @Override
        public int getNewId() {
            return newId;
        }

        @Override
        public FicheChange getFicheChange() {
            return ficheChange;
        }

        @Override
        public AttributeChange getAttributeChange() {
            return attributeChange;
        }

        @Override
        public LiensImport getLiensImport() {
            return liensImport;
        }

        @Override
        public FuzzyDate getCreationDate() {
            return creationDate;
        }

    }

}
